﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SyntaxTree
{

#region вспомогательное

    public enum Operation {plus, minus, mult, div, or, and, not, lt, gt, le, ge, eq, ne};
    public enum VarType { Integer, Double, Bool };

#endregion

    /// <summary>
    /// Базовый класс-предок всех узлов синтаксического дерева
    /// </summary>
    public  class SyntaxTreeNode
    {
        public SyntaxTreeNode()
        { }
        public virtual void visit(Visitor visitor)
        {
            visitor.visit(this);
        }
    }

    /// <summary>
    /// Класс-предок всех операторов
    /// </summary>
    public class Statement:SyntaxTreeNode
    {
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }
    /// <summary>
    /// Пустой оператор
    /// </summary>
    public class EmptyStatement : Statement
    {

        public EmptyStatement()
        {
            
        }
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }
    /// <summary>
    /// Оператор присваивания
    /// </summary>
    public class AssignStatement : Statement
    {
        public Expression From;
        public VarNode To;
        public AssignStatement(VarNode to, Expression from)
        {
            To = to;
            From = from;
        }
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }
    /// <summary>
    /// Цикл While
    /// </summary>
    public class WhileStatement : Statement
    {
        public Expression Condition;
        public Statement Body;
        public WhileStatement(Expression condition, Statement body)
        {
            Condition=condition;
            Body=body;
        }
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }

    /// <summary>
    /// Условный оператор
    /// </summary>
    public class IfStatement : Statement
    {
        public Expression Condition;
        public Statement ThenBody;
        public Statement ElseBody;
        public IfStatement(Expression condition, Statement thenbody)
        {
            Condition = condition;
            ThenBody = thenbody;
            
        }
        public IfStatement(Expression condition,Statement thenbody, Statement elsebody)
        {
            Condition=condition;
            ThenBody=thenbody;
            ElseBody=elsebody;
        }
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }

    /// <summary>
    /// Список операторов
    /// </summary>
    public class StatementList : Statement
    {
        public List<Statement> statements= new List<Statement>();
        public StatementList()
        {

        }
        public StatementList(List<Statement> statements)
        {
            this.statements = statements;
            
        }
        public void Add ( Statement st)
        {
            this.statements.Add(st);

        }
        public StatementList(Statement statement)
        {
            Add(statement);
            

        }
        public override void visit(Visitor visitor)
        {
            visitor.visit(this);
        }

    }
    /// <summary>
    /// Класс для представления главного блока программы. Корневой узел программы
    /// </summary>
    public class MainBlock : SyntaxTreeNode
    {
        public StatementList statements = new StatementList();
        public MainBlock()
        {

        }
        public MainBlock(StatementList statements)
        {
            this.statements = statements;

        }
        public static MainBlock Create(StatementList statements)
        {
            return new MainBlock(statements);
           

        }
        public void Add(Statement st)
        {
            this.statements.Add(st);

        }
        public MainBlock(Statement statement)
        {
            Add(statement);


        }
        public override void visit(Visitor visitor)
        {
            visitor.visit(this);
        }

    }
    /// <summary>
    /// Список переменных
    /// </summary>
   public class VarList : SyntaxTreeNode
	{

		public VarList()
		{

		}
		public VarList(List<VarNode> vars)
		{
			this.vars=vars;
		}

		private List<VarNode> vars=new List<VarNode>();
       		
		public List<VarNode> Vars
		{
			get
			{
				return vars;
			}
			set
			{
				vars=value;
			}
		}


		public void Add(VarNode var)
		{
		  vars.Add(var); //
		}
		
		public VarList(VarNode var)
		{
		  Add(var);
		}
		
		public void Add(string name)
		{
		  Add(new VarNode(name));
		}
		
		public VarList(string name)
		{
		  Add(name);
		}
       	public override void visit(Visitor visitor)
		{
			visitor.visit(this);
		}

	}
    /// <summary>
    /// Список переменных с индексом
    /// </summary>
   public class VarWithIndexList : SyntaxTreeNode
   {

       public VarWithIndexList()
       {

       }
       public VarWithIndexList(List<VarWithIndex> vars)
       {
           this.vars = vars;
       }

       private List<VarWithIndex> vars = new List<VarWithIndex>();

       public List<VarWithIndex> Vars
       {
           get
           {
               return vars;
           }
           set
           {
               vars = value;
           }
       }


       public void Add(VarWithIndex var)
       {
           vars.Add(var); //
       }

       public VarWithIndexList(VarWithIndex var)
       {
           Add(var);
       }

       public void Add(string name, Expression len)
       {
           Add(new VarWithIndex(name, len));
       }

       public VarWithIndexList(string name, Expression len)
       {
           Add(name,len);
       }
       public override void visit(Visitor visitor)
       {
           visitor.visit(this);
       }

   }

    /// <summary>
    ///  Базовый класс для выражений
    /// </summary>
    public  class Expression:SyntaxTreeNode
    {
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }
    /// <summary>
    /// Бинарное выражение
    /// </summary>
    public class BinaryExpression : Expression
    {
        public Expression Left;
        public Expression Right;
        public Operation Op;
        public BinaryExpression(Expression left, Expression right, Operation op)
        {
            Left = left;
            Right = right;
            Op = op;
        }
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }
    /// <summary>
    /// Унарное выражение
    /// </summary>
    public class UnaryExpression : Expression
    {
        public Expression Expr;
        public Operation Op;
        public UnaryExpression(Expression expr, Operation op)
        {
            Expr = expr;
            Op = op;
        }
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }
    /// <summary>
    /// Класс для представления переменной
    /// </summary>
    public class VarNode : Expression
    {
        public string Name;
        public VarNode(string nm)
        {
            this.Name=nm;
        }
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }
    
    /// <summary>
    /// Константное выражение
    /// </summary>
    public class ConstExpression : Expression
    {
        
        public ConstExpression()
        {
            
        }
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }
    /// <summary>
    /// Булева константа
    /// </summary>
    public class BoolConstExpression : ConstExpression
    {
        public bool value;
        public BoolConstExpression(bool val)
        {
            this.value = val;
        }
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }
    /// <summary>
    /// Целая константа
    /// </summary>
    public class IntConstExpression : ConstExpression
    {
        public int value;
        public IntConstExpression(int val)
        {
            this.value=val;
        }
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }
    /// <summary>
    /// Вещественная константа
    /// </summary>
    public class DoubleConstExpression : ConstExpression
    {
        public double value;
        public DoubleConstExpression(double val)
        {
            this.value=val;
        }
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }
    /// <summary>
    /// Переменная с индексом
    /// </summary>
    public class VarWithIndex : VarNode
    {        
        public Expression index;
        public VarWithIndex(string nm,Expression index):base(nm)
        {
           
            this.index=index;
                
        }
        public override void visit(Visitor vis)
        {
            vis.visit(this);
        }
    }
    /// <summary>
    /// Оператор определения переменной
    /// </summary>
    
    public class VarDefStatement : Statement
        {
            public VarType Type;
            public VarList Names = new VarList();
            public VarDefStatement(VarList names, VarType type)
            {
                this.Type = type;
                this.Names = names;
            }
            public override void visit(Visitor vis)
            {
                vis.visit(this);
            }
        }
    /// <summary>
    /// Оператор определения переменной с индексом
    /// </summary>
    public class VarWithIndexDefStatement : Statement 
        {
            public VarWithIndexList Vars = new VarWithIndexList();
            public VarType Type;
            public VarWithIndexDefStatement(VarWithIndexList Vars, VarType Type)
            {
                this.Vars = Vars;
                this.Type = Type;
            }
            public override void visit(Visitor vis)
            {
                vis.visit(this);
            }
        }
    
}
