﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;

namespace optimusParser
{
    // compile with: /doc:DocFileName.xml 
    /// <summary>Реализация интерфйеса IComparer<BaseBlock>. Позволяет сортировать базовые блоки по возрастанию их имен(имена задаются в виде чисел).</summary>
    class CompareBlock : IComparer<BaseBlock>
    {
        #region IComparer<BaseBlock> Members

        public int Compare(BaseBlock x, BaseBlock y)
        {
            return String.Compare(x.Name, y.Name);
        }

        #endregion
    }
    /// <summary>Реализация интерфйеса IEqualityComparer<BaseBlock>. Позволяет сравнивать ключи словаря необходимым нам образом. Объект данного класса будет передан констуктору объекта класса Dictionary<>.</summary>
    class BBlockComparer : IEqualityComparer<BaseBlock>
    {
        #region IEqualityComparer<BBlock> Members

        /// <summary>Функция сравнения двух объектов типа BaseBlock.</summary>
        /// <param name="x">Базовый блок.</param>         
        /// <param name="y">Базовый блок.</param>         
        /// <returns>Возвращает true, если имена базовых блоков равны, false в противном случае.</returns>
        public bool Equals(BaseBlock x, BaseBlock y)
        {
            return x.Name.Equals(y.Name);
        }

        /// <summary>Функция хеш-кода объекта типа BaseBlock.</summary>
        /// <param name="obj">Базовый блок.</param>                 
        /// <returns>Возвращает хеш-код имени базового блока.</returns>
        public int GetHashCode(BaseBlock obj)
        {
            return obj.Name.GetHashCode();
        }

        #endregion
    }

    /// <summary>Реализация интерфйеса BBGraph - графа базовых блоков. Негласное правило: Строковые имена блоков задаются в виде ПОЛОЖИТЕЛЬНЫХ ЦЕЛЫХ ЧИСЕЛ - необходимо для сортировки блоков в графе(Оптимизация работы итерационного алгоритма)</summary>
    class BaseBlockGraph : BBGraph
    {
        /// <summary>Приватное поле - список инцидентности.</summary>
        private Dictionary<BaseBlock, List<BaseBlock>> incidentList;
        /// <summary>Приватное поле - список имен базовых блоков в порядке добавления в граф.</summary>
        private List<string> NamesList;

        /// <summary>Конструктор инициализирует поля значениями по умолчанию.</summary>                                  
        public BaseBlockGraph()
        {
            BBlockComparer bcomparer = new BBlockComparer();
            incidentList = new Dictionary<BaseBlock, List<BaseBlock>>(bcomparer);
            NamesList = new List<string>();
        }

        #region BBGraph Members

        /// <summary>Индексное свойство на чтение/запись базового блока с именем name.</summary>
        /// <param name="name">Имя базового блока.</param>            
        /// <value>Тип значения BaseBlock.</value>
        public BaseBlock this[string name]
        {
            get
            {
                if (name != null)
                {
                    for (int i = 0; i < incidentList.Keys.Count; i++)
                        if (incidentList.Keys.ElementAt(i).Name.Equals(name))
                            return incidentList.Keys.ElementAt(i);
                    return null;                                        //если элемента с именем name не существует возвращем nuul
                }
                else throw new ArgumentNullException();
            }
            set
            {
                if (name != null)
                {
                    for (int i = 0; i < incidentList.Keys.Count; i++)
                        if (incidentList.Keys.ElementAt(i).Name.Equals(name))
                        {
                            List<BaseBlock> tempList = new List<BaseBlock>(incidentList[incidentList.Keys.ElementAt(i)]);
                            incidentList.Remove(incidentList.Keys.ElementAt(i));

                            if (value != null)
                            {
                                for (int j = 0; j < incidentList.Keys.Count; j++)
                                    if (incidentList.Keys.ElementAt(j).Name.Equals(value.Name))
                                        throw new Exception("Блок с таким же именем уже присутствует в графе!");
                                incidentList.Add(value, tempList);
                            }
                            else throw new ArgumentNullException();
                            break;
                        }
                }
                else throw new ArgumentNullException();
            }
        }

        /// <summary>Свойство на чтение количества вершин в графе.</summary>
        public int CountBBlock
        {
            get
            {
                return incidentList.Count;
            }
        }

        /// <summary>Свойство на чтение базовых блоков графа в виде списка List (список отсортирован в порядке возрастания имен базовых блоков).</summary>        
        public List<BaseBlock> ListBBlocks
        {
            get
            {
                List<BaseBlock> res = incidentList.Keys.ToList();
                CompareBlock cmp = new CompareBlock();
                res.Sort(cmp);
                return res;
            }
        }

        /// <summary>Свойство на чтение первого(добавленного в граф) базового блока. Если в графе нет элементов, возвращает null.</summary>
        public BaseBlock FirstBlocks
        {
            get
            {
                return this[NamesList.First()];
            }
        }

        /// <summary>Свойство на чтение последнего(добавленного в граф) базового блока. Если в графе нет элементов, возвращает null.</summary>
        public BaseBlock LastBlocks
        {
            get
            {
                return this[NamesList.Last()];
            }
        }

        /// <summary>Процедура добавляет вершину в граф.</summary>
        /// <param name="bblock">Базовый блок.</param>            
        public void AddBBlock(BaseBlock bblock) //добавить вершину в граф
        {
            if (bblock != null)
            {
                for (int i = 0; i < incidentList.Keys.Count; i++)
                    if (incidentList.Keys.ElementAt(i).Name.Equals(bblock.Name))
                        throw new Exception("Блок с таким же именем уже присутствует в графе!");
                incidentList.Add(bblock, new List<BaseBlock>());
                NamesList.Add(bblock.Name);
            }
            else throw new ArgumentNullException();
        }

        /// <summary>Процедура удаляет вершину из графа.</summary>
        /// <param name="name">Базовый блок.</param>            
        public void RemoveBBlock(string name)
        {
            if (name != null)
            {
                for (int i = 0; i < incidentList.Keys.Count; i++)
                    if (incidentList.Keys.ElementAt(i).Name.Equals(name))
                    {
                        for (int j = 0; j < incidentList.Keys.Count; j++)
                            this.RemoveEdge(name, incidentList.Keys.ElementAt(j).Name);
                        incidentList.Remove(incidentList.Keys.ElementAt(i));
                        NamesList.Remove(name);
                        break;
                    }
            }
            else throw new ArgumentNullException();
        }

        /// <summary>Процедура добавляет дугу между блоком c именем nameStartBBlock и блоком с именем nameEndBBlock.</summary>
        /// <param name="nameStartBBlock">Имя начального узла.</param> 
        /// <param name="nameEndBBlock">Имя конечного узла.</param> 
        public void AddEdge(string nameStartBBlock, string nameEndBBlock)
        {
            if (nameStartBBlock != null && nameEndBBlock != null)
            {
                BaseBlock start = null;
                BaseBlock end = null;
                for (int i = 0; i < incidentList.Keys.Count; i++)
                {
                    if (incidentList.Keys.ElementAt(i).Name.Equals(nameStartBBlock))
                        start = incidentList.Keys.ElementAt(i);
                    if (incidentList.Keys.ElementAt(i).Name.Equals(nameEndBBlock))
                        end = incidentList.Keys.ElementAt(i);
                }
                if (start == null || end == null)
                    return;
                incidentList[end].Add(start);
            }
            else throw new ArgumentNullException();
        }

        /// <summary>Процедура удаляет дугу между блоком c именем nameStartBBlock и блоком с именем nameEndBBlock.</summary>
        /// <param name="nameStartBBlock">Имя начального узла.</param> 
        /// <param name="nameEndBBlock">Имя конечного узла.</param> 
        public void RemoveEdge(string nameStartBBlock, string nameEndBBlock)
        {
            if (nameStartBBlock != null && nameEndBBlock != null)
            {
                BaseBlock start = null;
                BaseBlock end = null;
                for (int i = 0; i < incidentList.Keys.Count; i++)
                {
                    if (incidentList.Keys.ElementAt(i).Name.Equals(nameStartBBlock))
                        start = incidentList.Keys.ElementAt(i);
                    if (incidentList.Keys.ElementAt(i).Name.Equals(nameEndBBlock))
                        end = incidentList.Keys.ElementAt(i);
                }
                if (start == null || end == null)
                    return;
                incidentList[end].Remove(start);
            }
            else throw new ArgumentNullException();
        }

        /// <summary>Функция возвращает список всех предков узла с именем nameBBlock.</summary>
        /// <param name="nameBBlock">Имя узла.</param>         
        /// <returns>Возвращает список базовых блоков.</returns>        
        public List<BaseBlock> GetParentNodes(string nameBBlock)
        {
            if (nameBBlock != null)
            {
                BaseBlock block = null;
                for (int i = 0; i < incidentList.Keys.Count; i++)
                    if (incidentList.Keys.ElementAt(i).Name.Equals(nameBBlock))
                    {
                        block = incidentList.Keys.ElementAt(i);
                        break;
                    }
                if (block == null)
                    return new List<BaseBlock>();
                return incidentList[block];
            }
            else throw new ArgumentNullException();
        }

        /// <summary>Функция возвращает список всех потомков узла с именем nameBBlock.</summary>
        /// <param name="nameBBlock">Имя узла.</param>         
        /// <returns>Возвращает список базовых блоков.</returns>
        public List<BaseBlock> GetChildNodes(string nameBBlock)
        {
            if (nameBBlock != null)
            {
                List<BaseBlock> blist = new List<BaseBlock>();
                foreach (KeyValuePair<BaseBlock, List<BaseBlock>> kvp in incidentList)
                    for (int i = 0; i < kvp.Value.Count; i++)
                        if (kvp.Value[i].Name.Equals(nameBBlock))
                        {
                            blist.Add(kvp.Key);
                            break;
                        }
                return blist;
            }
            else throw new ArgumentNullException();
        }

        #endregion


    }
}
