﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace optimusParser
{
    // compile with: /doc:DocFileName.xml 
    /// <summary>Интерфйес базового блока.</summary>
    public interface BaseBlock : IEquatable<BaseBlock>
    {
        /// <summary>Свойство на чтение имени базового блока.</summary>
        /// <value>Тип значения string.</value>
        string Name
        {
            get;           
        }

        /// <summary>Свойство на чтение количества квадрик в блоке.</summary>
        int CountQuads 
        {
            get;
        }

        /// <summary>Индексное свойство на чтение/запись квадрики базового блока с номером index.</summary>
        /// <param name="index">Индекс квадрики.</param>            
        /// <value>Тип значения Quadric.</value>
        Quadric this[int index] 
        {
            get;
            set;
        }

        /// <summary>Свойство на чтение списка всех квадрик блока.</summary>
        List<Quadric> Quads 
        {
            get;
        }

        /// <summary>Процедура добаляет квадрику в конец блока.</summary>
        /// <param name="quad">Квадрика.</param>    
        void AddQuad(Quadric quad);

        /// <summary>Процедура вставляет квадрику quad в блок  в позицию index.</summary>
        /// <param name="index">Индекс квадрики.</param>    
        /// <param name="quad">Квадрика.</param>    
        void InsertQuad(int index, Quadric quad);

        /// <summary>Процедура удаляет по индексу квадрику из блока.</summary>
        /// <param name="index">Индекс квадрики.</param> 
        void RemoveQuad(int index); 
    }



    /// <summary>Интерфйес графа базовых блоков.</summary>    
    public interface BBGraph
    {
        /// <summary>Индексное свойство на чтение/запись базового блока с именем name.</summary>
        /// <param name="name">Имя базового блока.</param>            
        /// <value>Тип значения BaseBlock.</value>
        BaseBlock this[string name] //св-во для чтения/записи базового блока с именем name из графа
        {
            get;
            set;
        }

        /// <summary>Свойство на чтение количества вершин в графе.</summary>
        int CountBBlock //чтение количества вершин в графе
        {
            get;
        }

        /// <summary>Свойство на чтение базовых блоков графа в виде списка List.</summary>
        List<BaseBlock> ListBBlocks
        {
            get;
        }

        /// <summary>Свойство на чтение первого(добавленного в граф) базового блока.</summary>
        BaseBlock FirstBlocks
        {
            get;
        }

        /// <summary>Свойство на чтение последнего(добавленного в граф) базового блока.</summary>
        BaseBlock LastBlocks
        {
            get;
        }


        /// <summary>Процедура добавляет вершину в граф.</summary>
        /// <param name="bblock">Базовый блок.</param> 
        void AddBBlock(BaseBlock bblock); 

        /// <summary>Процедура удаляет вершину из графа.</summary>
        /// <param name="name">Базовый блок.</param> 
        void RemoveBBlock(string name);

        /// <summary>Процедура добавляет дугу между блоком c именем nameStartBBlock и блоком с именем nameEndBBlock.</summary>
        /// <param name="nameStartBBlock">Имя начального узла.</param> 
        /// <param name="nameEndBBlock">Имя конечного узла.</param> 
        void AddEdge(string nameStartBBlock, string nameEndBBlock);

        /// <summary>Процедура удаляет дугу между блоком c именем nameStartBBlock и блоком с именем nameEndBBlock.</summary>
        /// <param name="nameStartBBlock">Имя начального узла.</param> 
        /// <param name="nameEndBBlock">Имя конечного узла.</param> 
        void RemoveEdge(string nameStartBBlock, string nameEndBBlock);

        /// <summary>Функция возвращает список всех предков узла с именем nameBBlock.</summary>
        /// <param name="nameBBlock">Имя узла.</param>         
        /// <returns>Возвращает список базовых блоков.</returns>         
        List<BaseBlock> GetParentNodes(string nameBBlock);

        /// <summary>Функция возвращает список всех потомков узла с именем nameBBlock.</summary>
        /// <param name="nameBBlock">Имя узла.</param>         
        /// <returns>Возвращает список базовых блоков.</returns>
        List<BaseBlock> GetChildNodes(string nameBBlock);
    }
}
