﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace optimusParser
{
    // compile with: /doc:DocFileName.xml 
    /// <summary>Реализация интерфйеса BaseBlock - базового блока. Негласное правило: Строковые имена блоков задаются в виде ПОЛОЖИТЕЛЬНЫХ ЦЕЛЫХ ЧИСЕЛ - необходимо для сортировки блоков в графе(Оптимизация работы итерационного алгоритма).</summary>
    public class BBlock : BaseBlock    
    {
        /// <summary>Приватное поле - имя базового блока.</summary>
        private string name;
        /// <summary>Приватное поле - список квадрик.</summary>
        private List<Quadric> quads;

        /// <summary>Конструктор инициализирует приватные поля полями объекта block.</summary>
        /// <param name="block">Объект типа BBlock.</param>                
        public BBlock(BBlock block)
        {
            if (block != null)
            {
                this.name = block.Name;
                this.quads = new List<Quadric>(block.Quads);
            }
            else throw new ArgumentNullException();            
        }

        /// <summary>Конструктор инициализирует приватные поля именем name и списком квадрик quads.</summary>
        /// <param name="name">Параметр типа string.</param>                
        /// <param name="name">Объект типа List<Quadric>.</param>                
        public BBlock(string name, List<Quadric> quads)
        {
            if (name == null || name.Trim().Equals(""))
                throw new ArgumentNullException();
            else
            {
                this.name = name;
                if (quads != null)
                    this.quads = new List<Quadric>(quads);
                else this.quads = new List<Quadric>();
            }
        }

        /// <summary>Конструктор инициализирует приватное поле именем name.</summary>
        /// <param name="name">Параметр типа string.</param>                        
        public BBlock(string name)
        {
            if (name == null || name.Trim().Equals(""))
                throw new ArgumentNullException();
            else
            {
                this.name = name;
                this.quads = new List<Quadric>();
            }
        }
             
        #region BaseBlock Members

        /// <summary>Свойство на чтение имени базового блока.</summary>
        /// <value>Тип значения string.</value>
        public string Name 
        {
            get
            {                
                return name;
            }            
        }

        /// <summary>Свойство на чтение количества квадрик в блоке.</summary>
        public int CountQuads
        {
            get 
            {
                return quads.Count;                
            }
        }

        /// <summary>Индексное свойство на чтение/запись квадрики базового блока с номером index.</summary>
        /// <param name="index">Индекс квадрики.</param>            
        /// <value>Тип значения Quadric.</value>
        public Quadric this[int index] 
        {
            get
            {
                if (index < CountQuads && index >= 0)
                    return quads[index];
                else return null;                
            }
            set
            {
                if (index < CountQuads && index >= 0)
                    quads[index] = value;                
            }
        }

        /// <summary>Свойство на чтение списка всех квадрик блока.</summary>
        public List<Quadric> Quads
        {
            get
            {
                return quads;
            }
        }

        /// <summary>Процедура добаляет квадрику в конец блока.</summary>
        /// <param name="quad">Квадрика.</param>    
        public void AddQuad(Quadric quad) 
        {
            quads.Add(quad);
        }

        /// <summary>Процедура вставляет квадрику quad в блок  в позицию index.</summary>
        /// <param name="index">Индекс квадрики.</param>    
        /// <param name="quad">Квадрика.</param>    
        public void InsertQuad(int index, Quadric quad)
        {
            if (index < CountQuads && index >= 0)
                this.quads.Insert(index, quad);
            else throw new ArgumentOutOfRangeException();
        }

        /// <summary>Процедура удаляет по индексу квадрику из блока.</summary>
        /// <param name="index">Индекс квадрики.</param>            
        public void RemoveQuad(int index) 
        {
            if (index < CountQuads && index >= 0)
                quads.RemoveAt(index);
            else throw new ArgumentOutOfRangeException();
        }

        #endregion           
    
        #region IEquatable<BaseBlock> Members

        /// <summary>
        /// Функция проверяет блоки на равентсво. Блоки считаются равными если имена одинаковые.
        /// </summary>
        /// <param name="other">Базовый блок.</param> 
        /// <returns>Возвращает true, если имена равны, false в противном случае.</returns>
        public bool Equals(BaseBlock other) 
        {
            return this.Name.Equals(other.Name);            
        }

        #endregion
    }
}
