﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace optimusParser
{
    /// <summary>
    /// Используется для анализа графа потока управления.
    /// </summary>    
    class GraphAnalyser
    {
        //**********************************************************************************************
        // Олина часть
        //**********************************************************************************************
        /// <summary>
        /// Список раскрытых узлов.
        /// </summary>
        private static List<string> used;
        /// <summary>
        /// Счётчик для новой нумерации узлов.
        /// </summary>
        private static int cnt = 1;
        /// <summary>
        /// Графы.
        /// </summary>
        private static BaseBlockGraph bg, frame;

        /// <summary>
        /// Возвращает остовное дерево графа потока управления типа BaseBlock.
        /// </summary>
        /// <param name="bbg">Остовное дерево.</param>
        /// <returns>Новое остовное дерево.</returns>
        public static BaseBlockGraph SpanningTree(BaseBlockGraph bbg)
        {
            used = new List<string>();
            bg = bbg;
            frame = new BaseBlockGraph();  //создание нового графа-дерева
            AddToFrame(bg.FirstBlocks, null, bg.FirstBlocks.Name);
            return frame;
            
        }

        /// <summary>
        /// Рекурсивная процедура поиска в глубину, по графу потока управления bg строится новое дерево frame.
        /// </summary>
        /// <param name="bb">Имя добавляемой вершины в новом дереве.</param>
        /// <param name="par"> Родительский узел в дереве.</param>
        /// <param name="old_name">Имя узла в графе bg.</param>
        private static void AddToFrame(BaseBlock bb, BaseBlock par, string old_name)
        {
            //добавляем имя узла в графе bg в список раскрытых вершин
            used.Add(old_name);
            //добавляем блок в дерево 
            frame.AddBBlock(bb);
            //если данный блок – не root,
            //то добавляем в дерево связь между bb и его предком par
            if(par != null)
              frame.AddEdge(par.Name, bb.Name);
            List<BaseBlock> childs = bg.GetChildNodes(old_name);
            foreach (BaseBlock bbl in childs)
            {
                //если блок ещё не раскрыт, то
                if (!used.Contains(bbl.Name))
                {
                    //формируем блок дерева с новым именем
                    BaseBlock new_b = new BBlock(cnt.ToString(), bbl.Quads);
                    ++cnt;
                    //рекурсивно запускаем процедуру для всех дочерних вершин
                    AddToFrame(new_b, bb, bbl.Name);
                }
            }
        }

        //**********************************************************************************************
        // Машина часть
        //**********************************************************************************************

        private static BBGraph bbgraph;

        /// <summary>
        /// Получение доминаторов для графа.
        /// </summary>
        /// <param name="bbg">Граф.</param>
        /// <returns>Список доминаторов.</returns>
        public static List<HashSet<BaseBlock>> getDominators(BBGraph bbg)
        {
            bbgraph = bbg;
            List<HashSet<BaseBlock>> dom = new List<HashSet<BaseBlock>>(); // результирующий список всех доминаторов для каждой из вершин
            HashSet<BaseBlock> bBlocks = new HashSet<BaseBlock>(); // множество доминаторов для конкретной вершины
            List<BaseBlock> parentBlocks = new List<BaseBlock>();
            // D(1)={1}
            bBlocks.Add(bbg.ListBBlocks[0]);
            dom.Add(bBlocks);

            // заносим в множество доминаторов для каждой из вершин все вершины
            for (int i = 1; i < bbg.CountBBlock; ++i)
            {
                bBlocks = new HashSet<BaseBlock>();
                for (int j = 0; j < bbg.CountBBlock; ++j)
                    bBlocks.Add(bbg.ListBBlocks[j]);
                dom.Add(bBlocks);
            }

            for (int i = 1; i < bbg.CountBBlock; ++i)
            {
                // предшественники рассматриваемой вершины
                parentBlocks = bbg.GetParentNodes(bbg.ListBBlocks[i].Name);
                for (int j = 0; j < parentBlocks.Count; ++j)
                    // пересечение D(i), где i - все предшественники
                    dom[i].IntersectWith(dom[bbg.ListBBlocks.IndexOf(parentBlocks[j])]);
                HashSet<BaseBlock> tmpSet = new HashSet<BaseBlock>();
                tmpSet.Add(bbg.ListBBlocks[i]);
                // объединение пересечения с самой вершиной i
                dom[i].UnionWith(tmpSet);

            }
            return dom;
        }

        /// <summary>
        /// Проверяет является ли блок доминатором другого.
        /// </summary>
        /// <param name="d">Блок 1.</param>
        /// <param name="n">Блок 2.</param>
        /// <param name="g">Граф.</param>
        /// <returns>
        /// 	<c>true</c>если Блок 2 доминирует над Блок 1, иначе <c>false</c>.
        /// </returns>
        public static bool isDominate(BaseBlock d, BaseBlock n, BBGraph g)
        {
            bbgraph = g;
            // для каждой вершины строим множество доминаторов
            List<HashSet<BaseBlock>> dom = getDominators(g);
            List<BaseBlock> tmpList = g.ListBBlocks;
            // если D(n) содержит d, то d доминирует над n, иначе - нет
            return dom[tmpList.IndexOf(n)].Contains(d);
        }

        /// <summary>
        /// Вывод всех доминаторов..
        /// </summary>
        /// <param name="list">Список доминаторов.</param>
        public static void printDominators(List<HashSet<BaseBlock>> list)
        {
            Console.WriteLine("Dominators:");
            List<BaseBlock> tmpList = bbgraph.ListBBlocks;
            for (int i = 0; i < list.Count; ++i)
            {
                Console.Write("D(" + tmpList[i].Name + ") = ");
                foreach (BaseBlock b in list[i])
                {
                    Console.Write(b.Name + " ");
                }
                Console.WriteLine();
            }

        }

        //**********************************************************************************************
        // Женина и Катина часть
        //**********************************************************************************************

        /// <summary>
        /// Класс, описывающий ребро.
        /// </summary>
        internal class Edge
        {           
            /// <summary>
            /// Начало ребра.
            /// </summary>
            private BaseBlock beginBlock;          
            /// <summary>
            /// Конец ребра.
            /// </summary>
            private BaseBlock endBlock;
            /// <summary>
            /// Конструктор <see cref="Edge"/>.
            /// </summary>
            /// <param name="beginBlock">Начало ребра</param>
            /// <param name="endBlock">TКонец ребра.</param>
            public Edge(BaseBlock beginBlock, BaseBlock endBlock)
            {
                this.beginBlock = beginBlock;
                this.endBlock = endBlock;
            }

            /// <summary>
            /// Получаем начало ребра.
            /// </summary>
            /// <value>Начало ребра.</value>
            public BaseBlock BeginBlock
            {
                get
                {
                    return beginBlock;
                }
            }

            /// <summary>
            /// Получаем конец ребра.
            /// </summary>
            /// <value>Конец ребра.</value>
            public BaseBlock EndBlock
            {
                get
                {
                    return endBlock;
                }
            }

            public override string ToString()
            {
                return BeginBlock.Name + " " + EndBlock.Name;
            }
        }

        
        /// <summary>
        /// Обратные ребра.
        /// </summary>
        private List<Edge> backEdges = new List<Edge>();        
        /// <summary>
        /// Прямые ребра.
        /// </summary>
        private List<Edge> forwardEdges = new List<Edge>();       
        /// <summary>
        /// Отступающие ребра.
        /// </summary>
        private List<Edge> retreatingEdges = new List<Edge>(); 
        
        /// <summary>
        /// Классификация ребер.
        /// </summary>
        /// <param name="g">Граф.</param>
        public void EdgeClassification(BaseBlockGraph g)
        {
            // список посещенных вершин
            List<string> visited = new List<string>();
            // очередь для хранения вершин
            Queue<BaseBlock> queue = new Queue<BaseBlock>();
            // заносим в очередь первый блок
            queue.Enqueue(g.ListBBlocks[0]); 
            while (queue.Count != 0) // пока очередь не пуста
            {
                // извлекаем из нее очередной блок
                BaseBlock b = queue.Dequeue();
                // если он еше не был посещен
                if (!visited.Contains(b.Name)) 
                {
                    // пометить его как посещенный
                    visited.Add(b.Name);
                    // и пройтись по списку всех дочерних узлов
                    List<BaseBlock> childs = g.GetChildNodes(b.Name); 
                    for (int i = 0; i < childs.Count; ++i)
                    {
                        // если дочерний узел не находится в списке посещенных узлов
                        if (!visited.Contains(childs[i].Name)) 
                        {
                            // это прямое ребро
                            forwardEdges.Add(new Edge(b, childs[i]));
                            // и заносим дочерний узел в очередь
                            queue.Enqueue(childs[i]); 
                        }
                        // если дочерний узел уже был посещен, значит, это ребро отступающее
                        else 
                        {
                            retreatingEdges.Add(new Edge(b, childs[i]));
                            // если рассматриваемый дочерний узел доминирует над узлом b
                            if (isDominate(childs[i], b, g))
                                // то это отступающее ребро является и обратным
                                backEdges.Add(new Edge(b, childs[i])); 
                        }
                    }
                }
                //  иначе элемент без рассмотрения извлекается из очереди
            }
            // цикл завершится, так как добавление в очередь происходит только 
            //при рассмотрении прямых ребер, а их - конечное число
        }

        /// <summary>
        /// Получаем прямые ребра.
        /// </summary>
        /// <value>Прямые ребра.</value>
        public List<Edge> ForwardEges
        {
            get
            {
                return forwardEdges;
            }
        }
        /// <summary>
        /// Получаем обратные ребра.
        /// </summary>
        /// <value>Обратные ребра.</value>
        public List<Edge> BackEges
        {
            get
            {
                return backEdges;
            }
        }
        /// <summary>
        /// Получаем отступающие ребра.
        /// </summary>
        /// <value>Отступающие ребра.</value>
        public List<Edge> RetreatingEdges
        {
            get
            {
                return retreatingEdges;
            }
        }   
        /// <summary>
        /// Проверка графа на приводимость
        /// </summary>
        /// <param name="g">Граф.</param>
        /// <returns>
        /// 	<c>true</c> если граф приводим, иначе <c>false</c>.
        /// </returns>
        public bool isReducibleGraph(BBGraph g)
        {
            if (retreatingEdges.Count == backEdges.Count)
            {
                for (int i = 0; i < backEdges.Count; ++i)
                    if (!retreatingEdges.Contains(backEdges[i]))
                        return false;
                return true;
            }
            else
                return false;
        }
    }
}
