﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace optimusParser
{
    /// <summary>
    /// Класс для операций над графом.
    /// </summary>
    class GraphOperations
    {
        #region Fields

        /// <summary>
        /// Начальный граф.
        /// </summary>
        BaseBlockGraph originalGraph = null;

        /// <summary>
        /// Минимальное остовное дерево.
        /// </summary>
        BaseBlockGraph spanningTreeGraph = null;

        /// <summary>
        /// Матрица смежности между блоками начального графа.
        /// </summary>
        int[,] mtxAdj = null;

        /// <summary>
        /// Матрица смежности между блоками минимального остовного дерева.
        /// </summary>
        int[,] mtxAdjST = null;

        /// <summary>
        /// Матрица достижимости.
        /// </summary>
        int[,] mtxAtt = null;

        /// <summary>
        /// Количество вершин
        /// </summary>
        int vCount = 0;

        int[,] mtxFrame = null;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="GraphOperations"/> class.
        /// </summary>
        public GraphOperations(BaseBlockGraph graph)
        {
            originalGraph = graph;
            //Получаем список блоков
            var block_list = originalGraph.ListBBlocks;
            vCount = block_list.Count;
            //Инициализируем матрицу смежности
            mtxAdj = new int[vCount, vCount];
            //Для каждого блока
            for (int i = 0; i < block_list.Count; i++)
            {
                //Получаем его предков
                var parent_blocks = originalGraph.GetParentNodes(block_list[i].Name);
                //Заполняем матрицу
                try
                {
                    foreach (BaseBlock b in parent_blocks)
                        mtxAdj[Int32.Parse(b.Name), Int32.Parse(block_list[i].Name)] = 1;//Вот очень бы хотелось, чтобы так и было. А не так как при выводе в консоль...
                }
                catch (Exception e)
                {
                    Console.WriteLine("Error in matrix init. GraphOperations.CyclesFinder method. " + e.Message);
                }
            }
            //PrintMatrix();
            //Console.WriteLine();
        }

        #endregion

        #region Methods

        /// <summary>
        /// Вывод матрицы смежности.
        /// </summary>
        public void PrintMatrix()
        {
            for (int i = 0; i < vCount; i++)
            {
                for (int j = 0; j < vCount; j++)
                    Console.Write(mtxAdj[i, j] + " ");
                Console.WriteLine();
            }
        }

        /// <summary>
        /// Вывод матрицы смежности остовного дерева.
        /// </summary>
        public void PrintMatrixST()
        {
            for (int i = 0; i < vCount; i++)
            {
                for (int j = 0; j < vCount; j++)
                    Console.Write(mtxAdjST[i, j] + " ");
                Console.WriteLine();
            }
        }

        /// <summary>
        /// Вывод конкретной матрицы.
        /// </summary>
        /// <param name="matrix">Матрица.</param>
        public void PrintMatrixParam(ref int[,] matrix)
        {
            for (int i = 0; i < vCount; i++)
            {
                for (int j = 0; j < vCount; j++)
                    Console.Write(matrix[i, j] + " ");
                Console.WriteLine();
            }
        }

        // Метод, возвращающий матрицу достижимости
        public int[,] GetMatrixAttainability()
        {
            // Создаем матрицу смежности с рефлексивным отношением
            int[,] matrix = (int[,])mtxAdj.Clone();
            for (int i = 0; i < vCount; i++)
            {
                matrix[i, i] = 1;
            }

            // Создаем матрицу достижимости как матрицу смежности
            mtxAtt = matrix;

            // Запускаем цикл по количеству вершин
            for (int i = 0; i < vCount; i++)
            {
                // Умножаем матрицу достижимости на матрицу смежности
                mtxAtt = Multiplication(ref mtxAtt, ref matrix);
            }

            // "Нормируем" матрицу достижимости
            for (int i = 0; i < vCount; i++)
            {
                for (int j = 0; j < vCount; j++)
                {
                    // Если число матрицы отлично от 0
                    if (mtxAtt[i, j] != 0)
                    {
                        // Записываем 1 (то есть вершина достижима)
                        mtxAtt[i, j] = 1;
                    }
                }
            }

            // Возвращаем матрицу достижимости
            return mtxAtt;
        }

        /// <summary>
        /// Вычитание матриц.
        /// </summary>
        /// <param name="Matr1">Матрица 1.</param>
        /// <param name="Matr2">Матрица 2:Перезагрузка.</param>
        /// <returns>Новая матрица (Революция).</returns>
        public int[,] SubMatrix(ref int[,] Matr1, ref int[,] Matr2)
        {
            int[,] res = new int[vCount, vCount];
            try
            {
                for (int i = 0; i < vCount; i++)
                    for (int j = 0; j < vCount; j++)
                        res[i, j] = Matr1[i, j] - Matr2[i, j];
            }
            catch (ArithmeticException e)
            {
                Console.WriteLine("ArithmeticException in GraphOperations.SubMatrix. " + e.Message);
            }

            return res;
        }


        /// <summary>
        /// Метод перемножения матриц.
        /// </summary>
        /// <param name="Matr1">Матрица 1.</param>
        /// <param name="Matr2">Матрица 2:Перезагрузка.</param>
        /// <returns>Новая матрица (Революция).</returns>
        int[,] Multiplication(ref int[,] matrix1, ref int[,] matrix2)
        {
            // Создаем новую матрицу
            int[,] matrix = new int[vCount, vCount];

            // Цикл по количеству строк 1 матрицы
            for (int i = 0; i < matrix1.GetLength(0); i++)
            {
                // Цикл по количеству столбцов 2 матрицы
                for (int j = 0; j < matrix2.GetLength(1); j++)
                {
                    // Цикл по количеству строк 2 матрицы
                    for (int k = 0; k < matrix2.GetLength(0); k++)
                    {
                        // Получение [i,j] элемента итоговой матрицы
                        matrix[i, j] += matrix1[i, k] * matrix2[k, j];
                    }
                }
            }
            // Возвращаем результат перемножения матриц
            return matrix;
        }


        /// <summary>
        /// Проверка графа на связность.
        /// </summary>
        /// <returns>True- если связан, false- - иначе.</returns>
        bool CheckCoherence()
        {
            // Получаем матрицу достижиости
            int[,] matrix = GetMatrixAttainability();

            // Проверяем, связан ли граф
            foreach (int number in matrix)
            {
                // Если есть хоть один 0
                if (number == 0)
                    // Граф не связан
                    return false;
            }
            // Граф связан
            return true;
        }

       
        /// <summary>
        /// Находит циклы в графе.
        /// </summary>        
        /// <returns>1 - если успешно выполнено и 0 - иначе.</returns>
        public int CyclesFinder()
        {
            //Получаем минимальное остовное дерево
            spanningTreeGraph = SpanningTree.Create(originalGraph);
            //BBGraphVisualizator.CreatePNG(spanningTreeGraph);
            //Получаем список блоков
            var block_list = spanningTreeGraph.ListBBlocks;
            vCount = block_list.Count;
            //Инициализируем матрицу смежности дял дерева. Количество вершин должно совпадать с оригинальным графом.
            mtxAdjST = new int[vCount, vCount];
            int newVertName = 1;//Вот конечно, милый факт, что названия блоков изменились после создания дерева.
            System.Collections.Specialized.StringDictionary newBlockNames = new System.Collections.Specialized.StringDictionary();
            //Добавляем первый блок
            newBlockNames.Add("0", "0");
            //Для каждого блока
            for (int i = 0; i < block_list.Count; i++)
            {
                //Получаем его yfcktlybrjd
                var child_blocks = spanningTreeGraph.GetChildNodes(block_list[i].Name);
                //Заполняем матрицу
                try
                {
                    //Ставим в соответствие названия блоков согласно исходному графу
                    foreach (BaseBlock b in child_blocks)
                    {
                        newBlockNames.Add(b.Name, newVertName++.ToString());
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine("Error in matrix init. GraphOperations.CyclesFinder method. " + e.Message);
                    return 0;
                }
            }
            //Для каждого блока
            for (int i = 0; i < block_list.Count; i++)
            {
                //Получаем его предков
                var parent_blocks = spanningTreeGraph.GetParentNodes(block_list[i].Name);
                //Заполняем матрицу
                try
                {
                    foreach (BaseBlock b in parent_blocks)
                        mtxAdjST[Int32.Parse(newBlockNames[b.Name]), Int32.Parse(newBlockNames[block_list[i].Name])] = 1;
                }
                catch (Exception e)
                {
                    Console.WriteLine("Error in matrix init. GraphOperations.CyclesFinder method. " + e.Message);
                    return 0;
                }
            }
            int[,] cyclesMap = SubMatrix(ref mtxAdj, ref mtxAdjST);
            Console.WriteLine();
            //Ищем циклы...   
            for (int i = 0; i < vCount; i++)
                for (int j = 0; j < vCount; j++)
                    if (cyclesMap[i, j] == 1)
                    {
                        Console.WriteLine("Possible cycle: {0} from {1}. Is a cycle? {2}", j, i, IsCycle(j, i));
                        
                    }
            return 1;            
        }


        /// <summary>
        /// Определяем наличие цикла.
        /// </summary>
        /// <param name="dom">Доминатор цикла.</param>
        /// <param name="vert">Текущая вершина.</param>
        /// <returns>
        /// 	<c>true</c> если доминатор образует цикл над текущим ребром, иначе <c>false</c>.
        /// </returns>
        bool IsCycle(int dom, int vert)
        {
            if (dom == 0)
                return false;
            if (!CheckDom(dom, vert))
            {
                for (int i = 0; i < vCount; i++)
                    if (mtxAdj[i, vert] == 1)
                        IsCycle(i, dom);
            }
            return CheckDom(dom, vert);
        }

        /// <summary>
        /// Проверяет можно ли из Vert попасть в Dom.
        /// </summary>
        /// <param name="dom">Доминатор цикла.</param>
        /// <param name="vert">Текущая вершина.</param>
        /// <returns></returns>
        bool CheckDom(int dom, int vert)
        {
            for (int i = 0; i < vCount; i++)
                if (mtxAdj[i, vert] == 1 && i == dom)
                    return true;
            return false;
        }
        

        #endregion
    }
}
